-- $Id: lothlorien-wizards-spire.lua 3262 2018-03-03 20:40:12Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2018 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

local Quest = require 'engine.Quest'

store_util.set_prices(npc.owner, player, {
  id_pack = {900, 800, 100},
  recharge = {350, 300, 75},
})

local function fireproof_item(npc, player)
  local DamageType = require 'engine.DamageType'
  local q = player:hasQuest('lothlorien-fireproof')
  local cost = {
    ['magic/spellbook'] = 4,
    ['utility/staff'] = 3,
    ['utility/scroll'] = 1,
  }
  local function filter_cb(o)
    local key = o.type .. '/' .. o.subtype
    return cost[key] and not o:hasFlag('ignore_dam', DamageType.FIRE) and q.credits >= cost[key]
  end
  local function guts(o, inven, item, n)
    local key = o.type .. '/' .. o.subtype
    local oo, _ = player:removeObject(inven, item, n)
    local Ego = require 'mod.class.Ego'
    local fe = Ego:get('FIREPROOF')
    oo:forAllStack(function(fo)
      -- Fireproof is a simple ego, so we can get away with doing it like
      -- this.
      fe:resolveOnto(fo)
      table.set(fo, 'ego_names', 'prefix', 'FIREPROOF')
      return false
    end)
    player:addOrDropObject(inven, oo, true, nil, {vocal_add=true, vocal_drop=true})
    q.credits = q.credits - n * cost[key]
    player.changed = true
  end
  local function action(o, inven, item, btn, event)
    local key = o.type .. '/' .. o.subtype
    if o:getNumber() > 1 then
      local GetQuantity = require 'engine.dialogs.GetQuantity'
      local max = math.min(o:getNumber(), math.floor(q.credits/cost[key]))
      local function n_cb(qty)
	if qty then guts(o, inven, item, qty) end
      end
      game:registerDialog(GetQuantity.new('Fireproof item', '# to fireproof', 1, max, n_cb))
    else
      guts(o, inven, item, 1)
    end
    return true
  end
  player:showEquipInven('Fireproof item', filter_cb, action)
end

newChat {
  id = 'welcome1',
  text = store_util.price_interp([[#LIGHT_GREEN#*An aged and somewhat shabby-looking Elf looks up from a spellbook she is writing in and squints at you.*#LAST#
Hmm... @playerdescriptor.race@, yes?  I have trouble tellin' sometimes.  Rechargin's @price.recharge@ gold, ID's @price.id_pack@.  You do look like an adventurer, though, so maybe you can do somethin' for me.]]),
  answers = {
    { 'Depends.  What do you need?', jump='essence' },
    {
      'Identify my possessions, please.' .. store_util.cost_sfx('id_pack'),
      action = store_util.id_pack,
      cond = store_util.can_afford('id_pack'),
    },
    {
      'I have a wand/staff/rod I need recharged.' .. store_util.cost_sfx('recharge'),
      action = store_util.recharge_item,
      cond = store_util.can_afford('recharge'),
    },
    { 'Oh, sorry, I was looking for the potion store. [leave]' },
  },
}

newChat {
  id = 'essence',
  text = [[I'm workin' on a spell, and I need a very special essence to make it work.  There's one in a cave, up north of the spire.  I'd go fetch it myself, but I'm gettin' a bit old to be dealin' with all the lava and fire-breathin' creatures and whatnot.]],
  answers = {
    { '...A cave.  With lava.  In a forest.  How does #{italic}#that#{normal}# work exactly?', jump='cave' },
  },
}

newChat {
  id = 'cave',
  text = [[Now, never you mind that, young whippersnapper.  I'll make it worth your while, too.  I've got some fireproofin' reagent left over from back in my own adventurin' days, so those pesky fire breathers wouldn't burn up all my spellbooks; I can use `em to fix up your spellbooks for you too.  Or your staffs, if you're not the spellcastin' type.  Or scrolls, I guess, though I never saw the point in that m'self.]],
  answers = {
    { 
      "Sounds good.  I'm in.",
      action = function(npc, player)
	player:grantQuest('lothlorien-fireproof')
	util.place_quest_entrance('quest', 'mage', 'ENTER_LOTHLORIEN_FIREPROOF_QUEST', true)
      end,
    },
    { 'Bleh.  Never liked lava anyway. [leave]' },
  },
}

newChat {
  id = 'welcome2',
  text = [[#LIGHT_GREEN#*Valceronwe looks up from her spellbook as you enter.*#LAST#
Hmm... ah, right, you were getting my essence for me, weren't you?]],
  answers = {
    {
      'Yes, I found the eessence you wanted.  #LIGHT_GREEN#*give essence to Valceronwe*#LAST#',
      cond = function(npc, player)
	-- We should probably check that the player is actually carrying
	-- the essence...
	return player:isQuestStatus('lothlorien-fireproof', Quest.COMPLETED)
      end,
      action = function(npc, player)
	player:setQuestStatus('lothlorien-fireproof', Quest.COMPLETED, 'returned-essence')
	local o, item, inven = player:findInAllInventoriesBy('quest_obj', 'lothlorien-fireproof')
	if o then player:removeObject(inven, item, true) end
      end,
      jump = 'found',
    },
    {
      [[I'm sorry, I couldn't find the essence.]],
      cond = function(npc, player)
	return player:isQuestStatus('lothlorien-fireproof', Quest.FAILED)
      end,
      action = function(npc, player)
	player:setQuestStatus('lothlorien-fireproof', Quest.FAILED, 'returned-essence')
      end,
      jump = 'not-found',
    },
    {
      "Setting that aside for the moment, I'd like my possessions identified." .. store_util.cost_sfx('id_pack'),
      action = store_util.id_pack,
      cond = store_util.can_afford('id_pack'),
    },
    {
      'Setting that aside for the moment, I have a wand/staff/rod I need recharged.' .. store_util.cost_sfx('recharge'),
      action = store_util.recharge_item,
      cond = store_util.can_afford('recharge'),
    },
    {
      'I #{italic}#knew#{normal}# I was forgetting something... [leave]',
      cond = function(npc, player)
	return not player:isQuestStatus('lothlorien-fireproof', Quest.COMPLETED) and not player:isQuestStatus('lothlorien-fireproof', Quest.FAILED)
      end,
    },
  }
}

newChat {
  id = 'found',
  text = [[Ha!  Now I can #{italic}#finally#{normal}# get this blame fool spell workin'!  Oh, right, I promised you a reward, didn't I?  While you were out, I went diggin' around in the storeroom and found 12 vials of fireproofin' reagent that're still potent.  I figure it'll take four vials for a spellbook, three for a staff, or one for a scroll.  What'cha got for me?]],
  answers = {
    {
      "I have an item I'd like you to fireproof.",
      action = fireproof_item,
    },
    { 'Thank you; I will go and get something for you to fireproof. [leave]' },
  },
}

newChat {
  id = 'not-found',
  text = [[Meh, I'm sure I can talk someone else into findin' it.  Anything else I can do for ya while you're here?]],
  answers = {
    {
      'Identify my possessions, please.' .. store_util.cost_sfx('id_pack'),
      action = store_util.id_pack,
      cond = store_util.can_afford('id_pack'),
    },
    {
      'I have a wand/staff/rod I need recharged.' .. store_util.cost_sfx('recharge'),
      action = store_util.recharge_item,
      cond = store_util.can_afford('recharge'),
    },
    { "Nah, I'm good. [leave]" },
  },
}

newChat {
  id = 'welcome3',
  text = [[#LIGHT_GREEN#*Valceronwe looks up from her spellbook as you enter.*#LAST#
Hmm... @playername@, right?  I'll get the hang o' this memory thing yet, I tell ya.  So, got something for me to fireproof?]],
  answers = {
    { 'Yes, I do.', action=fireproof_item },
    { "...Well, I thought I did.  I'll be right back. [leave]" },
    {
      'Actually, could you identify my possessions, please?' .. store_util.cost_sfx('id_pack'),
      action = store_util.id_pack,
      cond = store_util.can_afford('id_pack'),
    },
    {
      'Actually, I have a wand/staff/rod I need recharged.' .. store_util.cost_sfx('recharge'),
      action = store_util.recharge_item,
      cond = store_util.can_afford('recharge'),
    },
  },
}

newChat {
  id = 'welcome4',
  text = [[#LIGHT_GREEN#*Valceronwe looks up from her spellbook as you enter.*#LAST#
Hmm... oh, right, @playername@.  What can I do ya fer?]],
  answers = {
    {
      'Identify my possessions, please.' .. store_util.cost_sfx('id_pack'),
      action = store_util.id_pack,
      cond = store_util.can_afford('id_pack'),
    },
    {
      'I have a wand/staff/rod I need recharged.' .. store_util.cost_sfx('recharge'),
      action = store_util.recharge_item,
      cond = store_util.can_afford('recharge'),
    },
    { 'Oh, sorry, I was looking for the potion store. [leave]' },
  },
}

if not player:hasQuest('lothlorien-fireproof') then
  return 'welcome1'
elseif player:isQuestStatus('lothlorien-fireproof', Quest.DONE) or player:isQuestStatus('lothlorien-fireproof', Quest.FAILED, 'returned-essence') then
  return 'welcome4'
elseif player:isQuestStatus('lothlorien-fireproof', Quest.COMPLETED, 'returned-essence') then
  return 'welcome3'
else
  return 'welcome2'
end
