-- $Id: quest-princess-finish.lua 2941 2017-06-20 02:16:32Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

local lev = game.zone:level_adjust_level(game.level, 'actor')
local state = game.state.princess_quests[lev]

local lastguy = state.n == 1 and state.name or 'the last '..state.name

newChat {
  id = 'begin',
  text = [[#LIGHT_GREEN#*As ]]..lastguy..[[ falls, the walls surrounding the princess vanish and a void jumpgate appears at her feet.*#LAST#
Oh, thank goodness.  With the cell's wards dispelled, my kingdom's wizards have finally found me.]],
  answers = {
    { 'Glad to be of service, Milady.', jump='reward' },
  },
}

local function place_reward(o, player, dlg)
  -- Remove the princess, and potentially place down stairs.
  local pr_spot = game.level:pickSpot{ type='princess-quest', subtype='princess' }
  local Map = require 'mod.class.Map'
  local princess = game.level.map(pr_spot.x, pr_spot.y, Map.ACTOR)
  local bottom = game.level.level == game.zone.max_level
  game:onTickEnd(function()
    game.level.map:remove(pr_spot.x, pr_spot.y, Map.ACTOR)
    -- TODO Remove the 'if' check once we figure out why princess quests
    -- aren't generating properly...
    if princess then
      game.level:removeEntity(princess)
      -- Put the princess back into circulation, so she can be generated
      -- for the next quest.
      princess:removed()
    end
    if not bottom then
      game.level.map(pr_spot.x, pr_spot.y, Map.TERRAIN, game.zone.grid_list.DOWN)
      -- Questionable Hack(TM):  The level probably doesn't have a
      -- default_down{}, since we blocked any down stairs; install this
      -- newly created down stair as default_down{} for the benefit of
      -- addons that make zones persistent.
      game.level.default_down = { x=pr_spot.x, y=pr_spot.y }
    end
  end)

  -- Place the reward object.
  game.zone:addEntity(game.level, o, 'object', pr_spot.x, pr_spot.y)
  o:setFoundHere {
    type = 'custom_with_level',
    custom = 'You received %s as a reward for rescuing a princess',
  }
  -- As a courtesy to the player, *ID* it when we place it.
  o:identify('full', player)

  -- Remember the reward object for the quest log.
  local q = game.player:hasQuest(state.q_id)
  if q then q.q_reward = o:getName() end

  -- Tweak the text of the last chat entry.
  local flat = game.level.data.generator.map.flags.FLAT
  local stair = bottom and '' or flat and ', revealing a path to the next area' or ', revealing the down stairs off the level'
  local it = o:getNumber() > 2 and 'them' or 'it'
  local chat = dlg.chat:get('finish')
  chat.text = [[Very well, I will leave ]]..it..[[ here for you.  Goodbye, and thank you again.  #LIGHT_GREEN#*The princess enters the void jumpgate, which closes behind her]]..stair..[[.*#LAST#]]
end

local reward_answers = {
  {
    'Then I choose the ' .. choices[1]:getName{ no_count=true, no_mods=true, no_pval=true } .. ', Milady.',
    action = function(npc, player, dlg) place_reward(choices[1], player, dlg) end,
    jump = 'finish',
  },
  {
    "I'd like the " .. choices[2]:getName{ no_count=true, no_mods=true, no_pval=true } .. ', then, please.',
    action = function(npc, player, dlg) place_reward(choices[2], player, dlg) end,
    jump = 'finish',
  },
  {
    'Ooh, that ' .. choices[3]:getName{ no_count=true, no_mods=true, no_pval=true } .. ' looks nice.',
    action = function(npc, player, dlg) place_reward(choices[3], player, dlg) end,
    jump = 'finish',
  },
}
if choices[3]:getNumber() > 1 then
  reward_answers[3][1] = 'Ooh, those ' .. choices[3]:getName{ no_count=true, no_mods=true, no_pval=true } .. ' look nice.'
end

newChat {
  id = 'reward',
  text = [[Before I go, I would like to reward you for rescuing me, noble Adventurer.  When my captors abducted me, they also stole these three magical items from our kingdom's treasure vaults.  I offer you any one of them, at your choice.]],
  answers = reward_answers,
}

newChat {
  id = 'finish',
  text = [[#RED#ERROR:  If you are seeing this text, something has gone wrong.#LAST#]],
  answers = {
    {
      'Thank you, Milady.',
      action = function(npc, player, dlg)
	player:setQuestStatus(state.q_id, engine.Quest.DONE)
      end,
    }
  },
}

return 'begin'
