-- $Id: last-alliance.lua 3479 2018-10-27 15:48:57Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2018 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

add_hooks = function()
  class:bindHook('CharacterSheet:dumpLine', function(self, data)
    local q = game.player:hasQuest('last-alliance')
    if q and q:isEnded() then
      table.insert(data.lines, [[You established a permanent void jumpgates liaison betwen Minas Anor and Gondolin,]])
      table.insert(data.lines, [[  thus allowing the last alliance to exist.]])
    end
  end)
end

id = 'last-alliance'
name = 'The Last Alliance'
reward = 'something'	-- populated by on_status_change() below

desc = function(self, who)
  if self:isEnded() then
    return ([[You aided Aragorn and Turgon in creating a long-range void jumpgate connecting Minas Anor and Gondolin.  As a reward, Aragorn granted you free use of the jumpgate, and gave you %s.]]):format(self.reward)
  elseif self:isCompleted('ambush-done') then
  return [[Aragorn of Minas Anor and Turgon of Gondolin are attempting to create a long-range void jumpgate connecting their two cities.  Turgon has given you the information they will need to complete the jumpgate; you must now return to the Castle in Minas Anor and deliver the information to Aragorn.  A wing of Thunderlords under the command of Trone who oppose the creation of the jumpgate ambushed you on the way, but you escaped them.]]
  elseif self:isCompleted('ambush') then
  return [[Aragorn of Minas Anor and Turgon of Gondolin are attempting to create a long-range void jumpgate connecting their two cities.  Turgon has given you the information they will need to complete the jumpgate; you must now return to the Castle in Minas Anor and deliver the information to Aragorn.  Unfortunately, you have been ambushed by a wing of Thunderlords under the command of Trone, who oppose the creation of the jumpgate and mean to stop you; you must slay all of them to escape the ambush.]]
  elseif self:isCompleted('gondolin') then
    return [[Aragorn of Minas Anor and Turgon of Gondolin are attempting to create a long-range void jumpgate connecting their two cities.  Turgon has given you the information they will need to complete the jumpgate; you must now return to the Castle in Minas Anor and deliver the information to Aragorn.  Be careful, though; a wing of Thunderlords under the command of Trone oppose the creation of the jumpgate and will probably try to stop you.]]
  else
    return [[Aragorn of Minas Anor and Turgon of Gondolin are attempting to create a long-range void jumpgate connecting their two cities.  You must go to the Tower of the King in Gondolin, where Turgon will give you the necessary information to complete the jumpgate.]]
  end
end

on_status_change = function(self, who, status, sub)
  if status == self.DONE then
    local o = game.zone:makeEntityByName(game.level, 'object', 'STAFF_GOLDEN_HORN_OF_THE_THUNDERLORDS')
    if o then
      o:identify(true, who)
      o:setFoundHere {
	type = 'custom',
	custom = 'You received %s as a reward for helping create a void jumpgate connecting Minas Anor and Gondolin.',
      }
      self.reward = o:getName()
      game.logPlayer(who, 'Aragorn gives you:  %s', o:getName())
      who:addOrDropObject(who.INVEN_INVEN, o, nil, nil, {vocal_drop=true})
      who:sortInven()
    end
  elseif status == self.COMPLETED and sub == 'ambush' then
    local Chat = require 'engine.Chat'
    -- A dummy NPC to display
    local npc = ui_util.chat_npc('GOLD_THUNDERLORD', {
      name = 'Thunderlord Captain',
      faction = 'enemies',
    })
    local chat = Chat.new('last-alliance-ambush', npc, game.player)
    ui_util.invoke_chat(chat)
  end
end

on_quest_mon_die = function(self, who, src)
  -- Only interested in the quest zone
  if game.zone.short_name ~= 'quest-last-alliance-ambush' then return end
  -- If any hostiles are still alive, we're not done.
  for _, e in pairs(game.level.entities) do
    if e.reactionToward and e:reactionToward(game.player) < 0 and not e.dead then
      return
    end
  end
  -- We can leave now; place the quest exit.
  game.log('#YELLOW#You can escape now.#LAST#')
  game.player:setQuestStatus('last-alliance', self.COMPLETED, 'ambush-done')
  local Map = require 'mod.class.Map'
  game.level.map(game.player.x, game.player.y, Map.TERRAIN, game.zone.grid_list.EXIT_LAST_ALLIANCE_AMBUSH)
end

callbacks = {
  on_move = function(who, q, moved, force, x, y, ox, oy)
    if who:isQuestStatus('last-alliance', q.COMPLETED, 'ambush') then
      -- If we've already done the ambush, punt.
      return
    end
    if game.zone and game.zone.short_name == 'quest-last-alliance-ambush' then
      -- If we're actually in the ambush, punt.
      return
    end
    if not who:isQuestStatus('last-alliance', q.COMPLETED, 'gondolin') then
      -- If we haven't gotten to Gondolin yet, punt.
      return
    end
    -- If we're not far enough south on the worldmap, punt.
    local wy = game.zone and game.zone.worldmap and y or who.wild_y
    if wy < 19 then return end
    -- Aaand here we go.
    game:changeLevel(1, 'quest-last-alliance-ambush')
  end,
}
