-- $Id: lothlorien-spiders.lua 3333 2018-04-20 00:32:38Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2018 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

id = 'lothlorien-spiders'
name = 'Spiders of Mirkwood'
danger_level = 25	-- cf. src/tables.c line 3872
reward = 'something'	-- populated by on_status_change() below

desc = function(self, who)
  if self:isEnded() then
    return ([[You slew the spiders encroaching on Lothlórien from Mirkwood.  As a reward, Celeborn gave you %s.]]):format(self.reward)
  elseif self:isCompleted() then
    return [[You have slain the spiders encroaching on Lothlórien from Mirkwood.  You should return to the Seat of Ruling and tell Celeborn.]]
  else
    return [[Spiders from Mirkwood are encroaching on Lothlórien from Mirkwood.  You must slay all of them.]]
  end
end

on_status_change = function(self, who, status, sub)
  if status == self.DONE then
    local o = game.zone:makeEntityByName(game.level, 'object', 'POT_AUGMENTATION')
    if o then
      o:identify(true, who)
      o:setFoundHere {
	type = 'custom',
	custom = 'You received %s as a reward for slaying the spiders of Mirkwood.',
      }
      self.reward = o:getName()
      game.logPlayer(who, 'Celeborn gives you:  %s', o:getName())
      who:addOrDropObject(who.INVEN_INVEN, o, nil, nil, {vocal_drop=true})
      who:sortInven()
    end
  elseif status == self.COMPLETED then
    if who.isFollowing and who:isFollowing('Yavanna') then
      -- Yavanna likes it when you save forests.
      game.logPlayer(who, '#LIGHT_GREEN#You feel the gentle touch of Yavanna as she smiles upon you.#LAST#')
      who:incPiety(6000)
    end
  end
end

on_quest_mon_die = function(self, who, src)
  -- Only interested in kills in the quest zone.
  if game.zone.short_name ~= 'quest-lothlorien-spiders' then return end
  -- If any hostiles are skill alive, we're not done.
  for _, e in pairs(game.level.entities) do
    if e.reactionToward and e:reactionToward(game.player) < 0 and not e.dead then
      return
    end
  end
  -- Our work here... is done.
  game.log('#YELLOW#The forest is safer now, thanks to you.#LAST#')
  game.player:setQuestStatus('lothlorien-spiders', self.COMPLETED)
end
