-- $Id: princess-sword-common.lua 2925 2017-06-02 03:06:25Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

-- Not a "quest" per se (so :grantQuest()'ing it would be probably be bad);
-- we merely install a hook to generate quest data common to the Princess
-- and Lost Sword quests.

add_hooks = function()
  class:bindHook('Birther:newPlayer', function(self, data)
    if game.state.princess_quests then return end

    -- Populate princess/lost-sword quests.
    game.state.princess_quests = {}
    game.state.sword_quests = {}
    local n_quests = game.state.birth.n_quests
    local mons = require('mod.class.Actor'):loadList('/data/general/npcs/all.lua')

    -- cf. src/birth.c lines 1253-1307 and monster_quest() in src/monster1.c
    local function qmon(e, lev, unique)
      -- No special monsters.
      if e:has('SPECIAL_GENE') then return false end
      -- No aquatic monsters.
      if e:has('AQUATIC') then return false end
      -- No breeders.
      if e:has('MULTIPLY') then return false end
      -- Only monsters that are allowed in the dungeon.
      if e.placement_flags.WILD_ONLY then return false end
      -- No friendly monsters.
      if e:has('PET') then return false end
      -- No Nazgul.
      if e:has('NAZGUL') then return false end
      -- No good monsters.
      if e:has('GOOD') then return false end
      -- No monsters with EXPLODE attacks.
      local AttackType = require 'mod.class.AttackType'
      for _, atk in ipairs(e.attacks or {}) do
	if atk.method == AttackType.EXPLODE then return false end
      end
      -- No uniques monsters --- unless we were asked for one, then only
      -- unique monsters..
      if not unique and e:has('UNIQUE') then return false end
      if unique and not e:has('UNIQUE') then return false end
      -- No monsters with a 'quest_on_die' field, since we're going to
      -- install our own.
      if e.quest_on_die then return false end
      -- Max 11 levels OoD; min 1 level OoD, with a max of 49
      -- (cf. RQ_LEVEL_CAP in src/birth.c).
      if e.level > lev+11 or e.level < math.min(lev+1, 49) then return false end

      return true
    end

    for i = 1, n_quests do
      local lev = math.floor(i * 98/n_quests)
      local info

      -- 1 in 4 chance of Lost Sword quest.
      if rng.chance(4) then
	local n = rng.table{20, 13, 15, 16, 9, 17, 18, 8}
	info = { n=n }
	game.state.sword_quests[lev] = info
      else
	local n = rng.table{1, 5, 6, 7, 10, 11, 12, 14}
	info = { n=n }
	game.state.princess_quests[lev] = info
      end

      -- Choose a quest monster suitable for this level.
      local cands = {}
      for _, m in ipairs(mons) do
	if qmon(m, lev, info.n == 1) then cands[#cands+1] = m end
      end
      if #cands > 0 then
	local qm = rng.table(cands)
	info.mon = qm.define_as
	info.name = qm:getName()
	info.pl_name = qm:getName{plural=true}
	info.pronoun = qm.unique and qm:pronoun('object') or 'them'
	print(('[QUEST] quest monster for level %d is %s'):format(lev, info.name))
      else
	error(('[QUEST] no candidate quest monsters for level '):format(lev))
      end
    end
  end)
end
