-- $Id: summons.lua 3187 2017-11-22 22:49:21Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

local Entity = require 'engine.Entity'

newTalentType {
  name = 'NPC summons',
  type = 'npc/summon',
  description = 'Summoning spells that can be cast by monsters',
}

newTalent {
  name = 'Summon Animal',
  short_name = 'NPC_S_ANIMAL',
  display_entity = Entity.new { image = 'talents/summon_animal.png' },
  type = 'npc/summon',
  mode = 'activated',
  points = 0,
  player = { mana=30 },
  range = 18,
  requires_target = true,
  direct_hit = true,
  innate = true,	-- cf. spell.fail_prob()
  message = false,
  action = spell.npc.summon_action,
  info = spell.npc.summon_info,
  params = {
    msg = '@Source@ magically summons an animal!',
    info_desc = 'an animal',
  },
  filter = { not_properties={'unique'}, deep_flags={flags={ANIMAL=true}} },
}

newTalent {
  name = 'Summon Animals',
  short_name = 'NPC_S_ANIMALS',
  display_entity = Entity.new { image = 'talents/summon_animals.png' },
  type = 'npc/summon',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=60, major=true },
  range = 18,
  requires_target = true,
  direct_hit = true,
  message = false,
  action = spell.npc.summon_action,
  info = spell.npc.summon_info,
  params = {
    num = 4,
    msg = '@Source@ magically summons some animals!',
    info_desc = 'animals',
  },
  filter = { not_properties={'unique'}, deep_flags={flags={ANIMAL=true}} },
}

newTalent {
  name = 'Summon Thunderlords',
  short_name = 'NPC_S_THUNDERLORD',
  display_entity = Entity.new { image = 'talents/summon_thunderlords.png' },
  type = 'npc/summon',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=90, major=true },
  range = 18,
  requires_target = true,
  direct_hit = true,
  message = false,
  action = spell.npc.summon_action,
  info = spell.npc.summon_info,
  params = {
    msg = '@Source@ magically summons a Thunderlord!',
    info_desc = 'a Thunderlord',
  },
  filter = { deep_flags={flags={THUNDERLORD=true}} },
}

newTalent {
  name = 'Summon Kin',
  short_name = 'NPC_S_KIN',
  display_entity = Entity.new { image = 'talents/summon_kin.png' },
  type = 'npc/summon',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=80 },
  range = 18,
  requires_target = true,
  direct_hit = true,
  message = false,
  action = spell.npc.summon_action,
  info = spell.npc.summon_info,
  params = {
    num = 6,
    msg = function(e)
      return '@Source@ summons @source.pronoun.possessive@ '..(e.unique and 'minions!' or 'kin!')
    end,
    info_desc = 'monsters of the same type as the caster',
  },
  filter = { kin = true },
}

newTalent {
  name = 'Summon Greater Demons',
  short_name = 'NPC_S_HI_DEMON',
  display_entity = Entity.new { image = 'talents/summon_greater_demons.png' },
  type = 'npc/summon',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=90, major=true },
  range = 18,
  requires_target = true,
  direct_hit = true,
  message = false,
  action = spell.npc.summon_action,
  info = spell.npc.summon_info,
  params = {
    num = function(e) return math.floor(game.level/50) + rng.range(0, 5) end,
    msg = '@Source@ magically summons greater demons!',
    info_desc = 'greater demons',
  },
  filter = { not_properties={'unique'}, deep_flags={flags={DEMON=true}}, display='U' },
}

newTalent {
  name = 'Summon Monster',
  short_name = 'NPC_S_MONSTER',
  display_entity = Entity.new { image = 'talents/summon_monster.png' },
  type = 'npc/summon',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=50 },
  range = 18,
  requires_target = true,
  direct_hit = true,
  message = false,
  action = spell.npc.summon_action,
  info = spell.npc.summon_info,
  params = {
    msg = '#Caster magically summons help!',
    info_desc = 'a monster',
  },
  filter = {},
}

newTalent {
  name = 'Summon Monsters',
  short_name = 'NPC_S_MONSTERS',
  display_entity = Entity.new { image = 'talents/summon_monsters.png' },
  type = 'npc/summon',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=60, major=true },
  range = 18,
  requires_target = true,
  direct_hit = true,
  message = false,
  action = spell.npc.summon_action,
  info = spell.npc.summon_info,
  params = {
    num = 8,
    msg = '#Caster magically summons monsters!',
    info_desc = 'several monsters',
  },
  filter = {},
}

newTalent {
  name = 'Summon Ants',
  short_name = 'NPC_S_ANT',
  display_entity = Entity.new { image = 'talents/summon_ants.png' },
  type = 'npc/summon',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=30 },
  range = 18,
  requires_target = true,
  direct_hit = true,
  message = false,
  action = spell.npc.summon_action,
  info = spell.npc.summon_info,
  params = {
    num = 6,
    msg = '@Source@ magically summons ants.',
    info_desc = 'ants',
  },
  filter = { not_properties={'unique'}, display='a' },
}

newTalent {
  name = 'Summon Spider',
  short_name = 'NPC_S_SPIDER',
  display_entity = Entity.new { image = 'talents/summon_spider.png' },
  type = 'npc/summon',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=30 },
  range = 18,
  requires_target = true,
  direct_hit = true,
  message = false,
  action = spell.npc.summon_action,
  info = spell.npc.summon_info,
  params = {
    num = 6,
    msg = '@Source@ magically summons spiders.',
    info_desc = 'ants',
  },
  filter = { not_properties={'unique'}, display='S' },
}

newTalent {
  name = 'Summon Hound',
  short_name = 'NPC_S_HOUND',
  display_entity = Entity.new { image = 'talents/summon_hound.png' },
  type = 'npc/summon',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=50, major=true },
  range = 18,
  requires_target = true,
  direct_hit = true,
  message = false,
  action = spell.npc.summon_action,
  info = spell.npc.summon_info,
  params = {
    num = 6,
    msg = '@Source@ magically summons hounds.',
    info_desc = 'hounds',
  },
  filter = { not_properties={'unique'}, displays='CZ' },
}

newTalent {
  name = 'Summon Hydra',
  short_name = 'NPC_S_HYDRA',
  display_entity = Entity.new { image = 'talents/summon_hydra.png' },
  type = 'npc/summon',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=40, major=true },
  range = 18,
  requires_target = true,
  direct_hit = true,
  message = false,
  action = spell.npc.summon_action,
  info = spell.npc.summon_info,
  params = {
    num = 6,
    msg = '@Source@ magically summons hydras.',
    info_desc = 'hydras',
  },
  filter = { not_properties={'unique'}, display='H' },
}

newTalent {
  name = 'Summon Angel',
  short_name = 'NPC_S_ANGEL',
  display_entity = Entity.new { image = 'talents/summon_angel.png' },
  type = 'npc/summon',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=60, major=true },
  range = 18,
  requires_target = true,
  direct_hit = true,
  message = false,
  action = spell.npc.summon_action,
  info = spell.npc.summon_info,
  params = {
    msg = '@Source@ magically summons an angel!',
    info_desc = 'an angel',
  },
  filter = { not_properties={'unique'}, display='A' },
}

newTalent {
  name = 'Summon Demon',
  short_name = 'NPC_S_DEMON',
  display_entity = Entity.new { image = 'talents/summon_demon.png' },
  type = 'npc/summon',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=60, major=true },
  range = 18,
  requires_target = true,
  direct_hit = true,
  message = false,
  action = spell.npc.summon_action,
  info = spell.npc.summon_info,
  params = {
    msg = '@Source@ magically summons a demon!',
    info_desc = 'a demon',
  },
  filter = { not_properties={'unique'}, deep_flags={flags={DEMON=true}} },
}

newTalent {
  name = 'Summon Undead',
  short_name = 'NPC_S_UNDEAD',
  display_entity = Entity.new { image = 'talents/summon_undead.png' },
  type = 'npc/summon',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=70, major=true },
  range = 18,
  requires_target = true,
  direct_hit = true,
  message = false,
  action = spell.npc.summon_action,
  info = spell.npc.summon_info,
  params = {
    msg = '@Source@ magically summons an undead adversary!',
    info_desc = 'an undead monster',
  },
  filter = { not_properties={'unique'}, deep_flags={flags={UNDEAD=true}} },
}

newTalent {
  name = 'Summon Dragon',
  short_name = 'NPC_S_DRAGON',
  display_entity = Entity.new { image = 'talents/summon_dragon.png' },
  type = 'npc/summon',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=70, major=true },
  range = 18,
  requires_target = true,
  direct_hit = true,
  message = false,
  action = spell.npc.summon_action,
  info = spell.npc.summon_info,
  params = {
    msg = '@Source@ magically summons a dragon!',
    info_desc = 'a dragon',
  },
  filter = { not_properties={'unique'}, deep_flags={flags={DRAGON=true}} },
}

newTalent {
  name = 'Summon High Undead',
  short_name = 'NPC_S_HI_UNDEAD',
  display_entity = Entity.new { image = 'talents/summon_high_undead.png' },
  type = 'npc/summon',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=90, major=true },
  range = 18,
  requires_target = true,
  direct_hit = true,
  message = false,
  action = spell.npc.summon_action,
  info = spell.npc.summon_info,
  params = {
    msg = '@Source@ magically summons greater undead!',
    info_desc = 'a powerful undead monster',
  },
  filter = { displays='LWV' },
}

newTalent {
  name = 'Summon High Dragon',
  short_name = 'NPC_S_HI_DRAGON',
  display_entity = Entity.new { image = 'talents/summon_high_dragon.png' },
  type = 'npc/summon',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=90, major=true },
  range = 18,
  requires_target = true,
  direct_hit = true,
  message = false,
  action = spell.npc.summon_action,
  info = spell.npc.summon_info,
  params = {
    num = 8,
    msg = '@Source@ magically summons ancient dragons!',
    info_desc = 'ancient dragons',
  },
  filter = { display='D' },
}

newTalent {
  name = 'Summon Wraith',
  short_name = 'NPC_S_WRAITH',
  display_entity = Entity.new { image = 'talents/summon_wraith.png' },
  type = 'npc/summon',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=90, major=true },
  range = 18,
  requires_target = true,
  direct_hit = true,
  message = false,
  action = spell.npc.summon_action,
  info = spell.npc.summon_info,
  params = {
    num = 8,
    msg = '@Source@ magically summons Wraith!',
    info_desc = 'wraiths',
  },
  filter = { display='W' },
  -- Annoying Hack(TM):  Double whammy...
  params2 = { num = 12 },
  filter2 = { displays='LWV' },
}

newTalent {
  name = 'Summon Unique',
  short_name = 'NPC_S_UNIQUE',
  display_entity = Entity.new { image = 'talents/summon_unique.png' },
  type = 'npc/summon',
  mode = 'activated',
  is_spell = true,
  points = 0,
  range = 18,
  requires_target = true,
  direct_hit = true,
  message = false,
  action = spell.npc.summon_action,
  info = spell.npc.summon_info,
  params = {
    num = 8,
    msg = '@Source@ magically summons special opponents!',
    info_desc = 'a unique creature (and some undead escorts)',
  },
  filter = { unique=true },
  -- Annoying Hack(TM):  Double whammy...
  params2 = { num = 8 },
  filter2 = { displays='LWV' },
}
