-- $Id: misc.lua 2837 2017-04-11 23:37:30Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

local Entity = require 'engine.Entity'

newTalentType {
  name = 'Miscellaneous talents',
  type = 'player/misc',
  description = [[Various player talents that don't obviously fit anywhere else]],
}

newTalent {
  name = 'Auto-Search',
  short_name = 'AUTO_SEARCH',
  display_entity = Entity.new { image='talents/auto_search.png' },
  sust_display = function(p) return 'Searching' end,
  type = 'player/misc',
  mode = 'sustained',
  info = function(who, t)
    return [[Automatically search adjacent grids for traps and secret doors (equivalent to one turn of 'S'earching) each time you move or stand still for a turn, at the cost of -10 speed.]]
  end,
  sustain_describe = function(who, t, p)
    return tstring{[[You are moving slowly (-10 speed) and searching adjacent grids for traps and secret doors as you move.]]}
  end,
  activate = function(who, def)
    game.logPlayer(who, 'You begin carefully examining your surroundings.')
    return {}
  end,
  deactivate = function(who, def)
    game.logPlayer(who, 'You stop examining your surroundings quite so carefully.')
    return true
  end,
  apply = function(who)
    who.speed = who.speed - 10
  end,
}

newTalent {
  name = 'Grow Mold',
  short_name = 'GROW_MOLD',
  display_entity = Entity.new { image='talents/grow_mold.png' },
  type = 'player/misc',
  mode = 'activated',
  power = { level=1, stat='CON', diff=14 },
  mana = 6,
  points = 0,
  action = function(self, t)
    local filter = { not_properties={'unique'}, display='m' }
    spell.summon(self, self.x, self.y, filter, 8)
    return true
  end,
  info = function(who, t)
    return [[Causes several molds to grow near you.]]
  end,
  self_knowledge_line = 'You can cause mold to grow near you.',
}
