-- $Id: abilities.lua 2835 2017-04-11 00:13:59Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org
  
newTalentType {
  name = 'Ability powers',
  type = 'skill/ability',
  description = 'Talents conferred by bought abilities',
}

local Entity = require 'engine.Entity'

-- This is handled by the Forge Shots/Arrows/Bolts talents.
--[=[
newTalent {
  name = [[Forge Ammo]],
  short_name = 'FORGE_AMMO',
  display_entity = Entity.new { image='talents/forge_ammo.png' },
  type = 'skill/ability',
  mode = 'activated',
  points = 0,
  action = function(self, t)
    game.log('Sorry, not implemented (FORGE_AMMO)')
  end,
  info = [[Watch this space]],
}
]=]

newTalent {
  name = [[Touch of Death]],
  short_name = 'TOUCH_OF_DEATH',
  display_entity = Entity.new { image='talents/touch_of_death.png' },
  type = 'skill/ability',
  mode = 'activated',
  points = 0,
  action = function(self, t)
    game.log('Sorry, not implemented (TOUCH_OF_DEATH)')
  end,
  info = [[Watch this space]],
}

newTalent {
  name = [[Far-Reaching Attack]],
  short_name = 'POLEARM_FAR',
  display_entity = Entity.new { image='talents/far_reaching_attack.png' },
  type = 'skill/ability',
  mode = 'activated',
  points = 0,
  action = function(self, t)
    game.log('Sorry, not implemented (POLEARM_FAR)')
  end,
  info = [[Watch this space]],
}

newTalent {
  name = [[Set Trap]],
  short_name = 'SET_TRAP',
  display_entity = Entity.new { image='talents/set_trap.png' },
  type = 'skill/ability',
  mode = 'activated',
  points = 0,
  callbacks = {
    on_object_desc = function(who, t, o, df, p)
      -- Describe what this object does if used in a trap.
      if t.chardump then return end	-- Except in a character dump
      if not o:isIDd() then return end	-- Or if it's not identified.
      local spec, ok
      if o.type == 'utility' and o.subtype == 'rod' then
	spec, ok = table.get(o, 'rod_tip', 'in_trap')
      else
	spec, ok = o.in_trap, true
      end
      if not ok or not spec then return end
      local descs = {}
      if spec.dam then
	local dam_desc = tostring(spec.dam)
	if type(spec.dam) == 'table' then
	  dam_desc = ('%dd%d'):format(spec.dam[1], spec.dam[2])
	end
	dam_desc = ('%s %s damage'):format(dam_desc, util.colorize_damtype(spec.damtype))
	if spec.r > 0 then
	  dam_desc = dam_desc .. (' in radius %d'):format(spec.r)
	end
	descs[#descs+1] = dam_desc
      end
      if spec.side_effect_desc then
	descs[#descs+1] = spec.side_effect_desc
      end
      descs = table.concat(descs, ' and ')
      p.desc = p.desc .. '\n#{bold}#When used in trap:#{normal}# ' .. descs
    end,
  },
  on_pre_use = function(self, t, silent, dry_run)
    local Map = require 'mod.class.Map'
    local map = game.level and game.level.map
    if not map then return false end
    if self:has('BLIND') then
      if not silent and not dry_run then
	game.logPlayer(self, 'You cannot set traps while blinded.')
      end
      return false
    end
    if not map.seens(self.x, self.y) then
      if not silent and not dry_run then
	game.logPlayer(self, 'It is too dark to set a trap.')
      end
      return false
    end
    if self:has('CONFUSED') then
      if not silent and not dry_run then
	game.logPlayer(self, 'You cannot set traps while confused.')
      end
      return false
    end
    if not map:checkEntity(self.x, self.y, Map.TERRAIN, 'clean_bold') then
      if not silent and not dry_run then
	game.logPlayer(self, 'You cannot set a trap here.')
      end
      return false
    end
    local ret, _ = self:findInInventoryBy(self:getInven('INVEN'), 'subtype', 'trapkit')
    if not ret and not silent and not dry_run then
      game.logPlayer(self, 'You have no trap kits to use.')
    end
    return ret
  end,
  action = function(self, t)
    return trap_util.set_trap(self, t)
  end,
  info = [[Set a trap at your current location using a trap kit.  Player-set traps will be triggered by hostile monsters.  The trap kit can be retrieved and repositioned by disarming the trap.]],
}
