-- $Id: zone.lua 2835 2017-04-11 00:13:59Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

-- Zone file for Cirith Ungol.

local DamageType = require('engine.DamageType')

return {
  name = 'Cirith Ungol',
  display_abbr = 'CUg',
  level_scheme = 'fixed',
  max_level = 26,
  level_range = { 25, 50 },
  width = 198,
  height = 66,
  ambient_damage = {
    damage = { 4, 4 },
    frequency = 20,
    damtype = DamageType.POISON,
    kill_text = [[the toxic vapors of Cirith Ungol]],
  },

  generator = {
    map = {
      class = 'mod.class.generator.map.T2DungeonLevel',
      floor = { weighted = { DARK_PIT=5, DIRT=65, WEB=30 } },
      fill = { weighted = { MOUNTAIN_CHAIN=90, WEB=10 } },
      outer_wall = 'WEB',
      inner_wall = 'WALL',
      fill_step = 2,
      up = 'UP',
      down = 'DOWN',
      flags = {
	CIRCULAR_ROOMS = true,
      },
    },
    actor = {
      class = 'mod.class.generator.actor.T2LevelFilteredActors',
      init_npc = 14,
      respawn_in = 160,
      filters = {
	{ prob=2, mode='any' },
	{ prob=49, mode='or', syms={ 'c', 'a', 'I' }, flags={ 'SPIDER' } },
	{ prob=49, mode='or', syms={ 'w', 'm', 'j' }, flags={ 'ORC' } },
      },
      guardian = 'SHELOB_SPIDER_OF_DARKNESS',
    },
    object = {
      class = 'mod.class.generator.object.T2LevelFilteredObjects',
      weights = { treasure=30, combat=30, magic=30, tools=10, junk=0 },
    },
    trap = {
      class = 'mod.class.generator.trap.T2LevelFilteredTraps',
    },
  },
  alter_level_data = util.adjust_level,
  levels = {
    [1] = {
      generator = {
	map = {
	  up = 'UP_TO_WILDERNESS',
	  -- up_lev = 1
	},
      },
    },
  },
}
