-- $Id: zone.lua 3333 2018-04-20 00:32:38Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2018 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

return {
  name = "Dragons' Nest",
  level_scheme = 'fixed',
  max_level = 1,
  level_range = { 25, 25 },
  width = 33,
  height = 23,
  no_level_connectivity = true,
  no_recall_depth = true,

  generator = {
    map = {
      class = 'mod.class.generator.map.Static',
      map = 'quests/gondolin-dragons',
      flags = { NO_GENO=true },
    },
    actor = {
      class = 'mod.class.generator.actor.T2LevelFilteredActors',
      -- We'll generate actors in post_process() below.
      no_prepop = true,
    },
  },

  -- cf. quest_dragons_gen_hook() in src/q_dragons.c
  post_process = function(level, zone)
    -- Place some columns
    local Map = require 'mod.class.Map'
    local map = level.map
    local n = 35
    while n > 0 do
      local x, y = rng.range(0, 30) + 1, rng.range(0, 20) + 1
      -- Bar columns on even squares so the whole level is guaranteed
      -- accessible.
      -- [The original T2 code does this wrong, I think; it checks for
      --  (flags%2)==0, where 'flags' is the 'flags1' grid flags field, so
      --  that has the effect of excluding grids with the NO_WALK flag ---
      --  which is probably redundant with requiring the FLOOR flag.  We
      --  instead require (x+y)%2==0, which will have a "checkerboardy"
      --  effect but will achieve the intended goal.]
      if (x + y) % 2 == 0 and map:checkEntity(x, y, Map.TERRAIN, 'floor') and not map:checkEntity(x, y, Map.TERRAIN, 'permanent') then
	map(x, y, Map.TERRAIN, zone.grid_list.MOUNTAIN_CHAIN)
	n = n - 1
      end
    end
    -- Place some dragons.
    local gen = zone:getGenerator('actor', level)
    local colors = {
      'BLUE', 'WHITE', 'RED', 'BLACK', 'BRONZE', 'GOLD', 'GREEN', 'MULTI_HUED',
    }
    n = 25
    while n > 0 do
      local x, y = rng.range(0, 30) + 1, rng.range(0, 20) + 1
      if map:checkEntity(x, y, Map.TERRAIN, 'floor') and not map:checkEntity(x, y, Map.TERRAIN, 'permanent') then
	local pct = rng.range(0, 99)
	local age = pct == 0 and 'ANCIENT' or pct < 33 and 'BABY' or pct < 66 and 'YOUNG' or 'MATURE'
	local color = rng.table(colors)
	local tag = age..'_'..color..'_DRAGON'
	local m = gen:generateOne{ name=tag, x=x, y=y, sleep=33, ret=true }
	if m then
	  m.quest_on_die = 'gondolin-house'
	  n = n - 1
	end
      end
    end
  end,
}
