-- $Id: zone.lua 3333 2018-04-20 00:32:38Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2018 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

local Dialog = require 'engine.ui.Dialog'

return {
  name = [[Deep Stacks]],
  level_scheme = 'fixed',
  max_level = 1,
  level_range = { 35, 35 },
  width = 65,
  height = 38,
  no_level_connectivity = true,
  no_recall_depth = true,

  generator = {
    map = {
      class = 'mod.class.generator.map.Static',
      map = 'quests/minas-anor-library',
      flags = { NO_GENO=true },
    },
    actor = {
      class = 'mod.class.generator.actor.T2LevelFilteredActors',
      -- We'll generate actors in post_process() below.
      no_prepop = true,
    },
  },

  post_process = function(level, zone)
    local gen = zone:getGenerator('actor', level)
    local function place_mons(miny, minx, maxy, maxx, name, n)
      local area = { x1=minx, x2=maxx, y1=miny, y2=maxy }
      while n > 0 do
	local m = gen:generateOne{ name=name, area=area, ret=true }
	if m then
	  m.quest_on_die = 'minas-anor-library'
	  n = n - 1
	end
      end
    end

    place_mons(2, 2, 12, 35, 'LICH', rng.dice(4, 2))
    place_mons(12, 32, 35, 65, 'MONASTIC_LICH', rng.dice(1, 2))
    place_mons(2, 32, 12, 65, 'MONASTIC_LICH', rng.dice(1, 2) - 1)
    place_mons(12, 2, 35, 32, 'MONASTIC_LICH', rng.dice(1, 2) - 1)
    place_mons(8, 8, 35, 65, 'FLESH_GOLEM', 2)
    place_mons(8, 8, 35, 65, 'CLAY_GOLEM', 2)
    place_mons(8, 8, 35, 65, 'IRON_GOLEM', 2)
    place_mons(8, 8, 35, 65, 'MITHRIL_GOLEM', 1)
  end,
}
