-- $Id: zone.lua 3307 2018-03-24 02:34:16Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2018 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

local Quest = require 'engine.Quest'

return {
  name = 'Gondolin',
  level_scheme = 'fixed',
  max_level = 1,
  width = 198,
  height = 66,
  no_level_connectivity = true,
  all_remembered = true,
  all_lited = true,
  day_night = true,
  town = true,

  generator = {
    map = {
      class = 'mod.class.generator.map.PlasmaWilderness',
      map = game.state:varTownDestroyed('gondolin', 'map'),
    },
    actor = game.state:varTownDestroyed('gondolin', 'actor_gen', {
      class = 'mod.class.generator.actor.T2LevelFilteredActors',
      init_npc = 10,
      filters = {
	{ prob=100, mode='any' },
      },
      zone_resolve_flags = { use_ego='ELVEN' },
    }),
  },

  entry_points_from_zone = {
    ['quest-gondolin-dragons'] = { type='quest', subtype='house' },
    ['quest-eol-the-dark-elf'] = { type='quest', subtype='eol' },
    ['quest-nirnaeth-arnoediad'] = { type='quest', subtype='nirnaeth' },
    ['quest-invasion-of-gondolin'] = { type='quest', subtype='invasion' },
    ['town-minas-anor'] = { type='quest', subtype='between-gate' },
  },
  entry_point = util.town_wild_entry_point,

  quest_entrances = {
    ['gondolin-house'] = {
      type='quest', subtype='house', replace={
	[Quest.PENDING] = 'ENTER_GONDOLIN_HOUSE_QUEST',
	[Quest.COMPLETED] = 'ENTER_GONDOLIN_HOUSE_QUEST',
	-- Default grid is PERMA_WALL, which is what we wanted anyway.
	[Quest.DONE] = { trap='GONDOLIN_HOME' },
      },
    },
    ['eol-the-dark-elf'] = {
      type='quest', subtype='eol', replace={
	[Quest.PENDING] = 'ENTER_GONDOLIN_EOL_QUEST',
	-- Let the entrance go away after we complete (or fail) the quest
      },
    },
    ['nirnaeth-arnoediad'] = {
      type='quest', subtype='nirnaeth', replace={
	[Quest.PENDING] = 'ENTER_GONDOLIN_NIRNAETH_QUEST',
	-- Let the entrance go away after we complete (or fail) the quest
      },
    },
    ['invasion-of-gondolin'] = {
      {
	type='quest', subtype='invasion', replace={
	  [Quest.PENDING] = {
	    if_completed='turgon', grid='ENTER_GONDOLIN_INVASION_QUEST',
	  }
	  -- Let the entrance go away after we complete (or fail) the quest
	},
      },
      {
	type='quest', subtype='jewelry', replace={
	  -- Default grid is PERMA_WALL, which is what we wanted anyway.
	  [Quest.DONE] = { trap='RARE_JEWELRY_SHOP' },
	},
      },
    },
    ['last-alliance'] = {
      type='quest', subtype='between-gate', replace={
	[Quest.DONE] = 'GONDOLIN_VOID_JUMPGATE',
      },
    },
  },
  on_enter = util.juggle_quest_entrances,

  on_leave = util.on_leave_town_wild,
}
