-- $Id: tome2.lua 3260 2018-03-03 20:17:33Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2018 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

-- cf. process_monsters() and process_monster() in src/melee2.c
newAI('tome2', function(self)
  -- Only process NPCs within 100 grids of the player.
  local plyr_dist = core.fov.distance(self.x, self.y, game.player.x, game.player.y)
  if plyr_dist >= 100 then
    -- NPC:act() will use energy for us.
    return true
  end

  local active = false
  -- TODO active if directly controlled by player
  -- TODO active if partial summon
  -- Process the monster if we're in its sense range...
  active = active or plyr_dist <= self.vision
  -- ...or if it has line of sight to us...
  active = active or self:hasLOS(game.player.x, game.player.y, 20)
  -- ...or if we're aggravating.
  active = active or game.player:has('AGGRAVATE')
  -- TODO "Monsters chase current location (v.slow)" option (ie. flow_by_sound)

  if not active then
    -- Aww, maybe next time...
    -- NPC:act() will use energy for us.
    return true
  end

  if self.ai_state.sleep > 0 then
    local notice = game.player and game.player:has('AGGRAVATE') and 0 or rng.range(0, 1023)
    print(('[DEBUG] notice %d^3 = %d vs. noise %d'):format(notice, notice*notice*notice, game.player.stealth_exp))
    if game.player and game.player.stealth_exp and notice*notice*notice <= game.player.stealth_exp then
      local d = math.max(1, math.floor(100/plyr_dist))
      self.ai_state.sleep = math.max(0, self.ai_state.sleep - d)
      if game.player:has('AGGRAVATE') then self.ai_state.sleep = 0 end
      if self.ai_state.sleep > 0 then
	-- Notice mon not waking up, for monster memory purposes.
	self:learnFlags('sleep', 'ignore')
	return
      end
      game.logSeen(self, "%s wakes up.", self.name)
      -- Notice mon waking up, for monster memory purposes.
      self:learnFlags('sleep', 'wake')
    end
  end

  if self.flags.MULTIPLY then
    -- Chance of multiplying depends on number of monsters next to us.
    local Map = require 'mod.class.Map'
    local x, y
    local n = 0
    for x = self.x - 1, self.x + 1 do
      for y = self.y - 1, self.y + 1 do
	if game.level.map(x, y, Map.ACTOR) then n = n + 1 end
      end
    end
    if n == 0 or (n < 4 and rng.chance(n*10)) then
      if spell.try_breed_monster(self, false, false) then
	self:learnFlags('flags', 'MULTIPLY')
	-- NPC:act() will use energy for us.
	return true
      end
    end
  end

  return self:runAI(self.ai_state.sub_ai or 'dumb_talented_simple')
end)

-- Movement AI for monsters with the AI_ANNOY flag.  Loosely based on the
-- engine's 'move_snake' AI.
newAI('tome2-move-annoy', function(self)
  if self.ai_target.actor then
    local tx, ty = self:aiSeeTargetPos(self.ai_target.actor)
    if self:isNear(tx, ty, 3) then
      -- If we're near the target, back up.
      local dir = util.getDir(tx, ty, self.x, self.y)
      tx, ty = util.coordAddDir(self.x, self.y, util.opposedDir(dir, self.x, self.y))
    end
    return self:moveDirection(tx, ty)
  end
end)

newAI('tome2_worn_symbiote', function(self)
  local host = self.symbiosis and self.symbiosis.host
  if not host or not host.x or host.dead then return end
  -- Don't do anything if we're not "live".
  if not config.settings.tome2.live_symbiote then return end

  -- Collect adjacent targets.
  local Map = require 'mod.class.Map'
  local tgts = {}
  for x = host.x - 1, host.x + 1 do
    for y = host.y - 1, host.y + 1 do
      local a = game.level.map(x, y, Map.ACTOR)
      -- Slight Hack(TM):  We'll assume our symbiosis extends to allowing
      -- us to see anything our host can.
      if a and not a.dead and self:reactionToward(a) < 0 and (self:canSee(a) or host:canSee(a)) then
	tgts[#tgts+1] = a
      end
    end
  end
  if #tgts > 0 then
    -- Pick a random target and attack it.
    local a = rng.table(tgts)
    if a and not a.dead then self:attackTarget(a) end
  end
  return true
end)
