-- $Id: AlchemyGoldPopup.lua 2870 2017-05-09 23:06:59Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

require 'engine.class'
local Dialog = require 'engine.ui.Dialog'
local Button = require 'engine.ui.Button'
local Textzone = require 'engine.ui.Textzone'
local Numberbox = require 'engine.ui.Numberbox'

module(..., package.seeall, class.inherit(Dialog))

local tmpl = [[This %s is difficult to %s at your current Alchemy skill level; there is a a %d%% chance of failure and explosive backfire.  You can decrease this chance by adding gold to the process (roughly %d gold per 1%% reduction).]]

function _M:init(actor, title, failprob, value, adding_ego)
  self.actor = actor
  Dialog.init(self, title, 1, 1)

  local item, create = 'item', 'create'
  if adding_ego then item, create = 'ego', 'add' end
  self.c_txt = Textzone.new {
    width = 600,
    auto_height = true,
    text = tmpl:format(item, create, failprob, math.ceil(value/20)),
  }
  self.c_num = Numberbox.new {
    title = 'Gold to add: ',
    number = 0,
    max = self.actor.gold,
    min = 0,
    chars = 10,
    fct = function(text) self:okclick() end,
  }
  self.c_ok = Button.new {
    text = 'Continue',
    fct = function(text) self:okclick() end,
  }
  self.c_cancel = Button.new {
    text = 'Do not create',
    fct = function(text) self:returnVal(-1) end,
  }

  self:loadUI {
    { left=0, top=0, ui=self.c_txt },
    { left=0, top=self.c_txt, padding_h=10, ui=self.c_num },
    { left=0, bottom=0, ui=self.c_ok },
    { right=0, bottom=0, ui=self.c_cancel },
  }
  self:setFocus(self.c_num)
  self:setupUI(true, true)

  self.key:addBinds {
    EXIT = function() self:returnVal(-1) end,
  }
end

function _M:returnVal(n)
  self.actor:talentDialogReturn(n)
  game:unregisterDialog(self)
end

function _M:okclick()
  local qty = self.c_num.number
  if qty then
    self:returnVal(qty)
  else
    Dialog:simplePopup('Error', 'Enter a quantity.')
  end
end
