-- $Id: ShowAchievements.lua 2835 2017-04-11 00:13:59Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

require 'engine.class'
local ShowAchievements = require 'engine.dialogs.ShowAchievements'
local Tiles = require "engine.Tiles"

module(..., package.seeall, class.inherit(ShowAchievements))

function _M:generateList(kind)
  local tiles = Tiles.new(16, 16, nil, nil, true)
  local cache = {}

  -- Makes up the list
  local list = {}
  local function handle(id, data)
    local a = world:getAchievementFromId(id)
    local color = nil
    if self.player and self.player.achievements and self.player.achievements[id] then
      color = colors.simple(colors.LIGHT_GREEN)
    end
    local img = a.image or "trophy_gold.png"
    local tex = cache[img]
    if not tex then
      local image = tiles:loadImage(img)
      if image then
	tex = {image:glTexture()}
	cache[img] = tex
      end
    end
    if not data.notdone or a.show then
      local item = {
	name = a.name,
	color = color,
	desc = a.desc,
	category = a.category or '--',
	when = data.when,
	who = data.who,
	order = a.order,
	id = id,
	tex = tex,
	a = a,
      }
      if a.show == "full" or not data.notdone then
	if a.desc_undone and data.notdone then item.desc = a.desc_undone end
      elseif a.show == "none" then
	item.name, item.desc = '???', '-- Unknown --'
      elseif a.show == "name" then
	item.desc = '-- Unknown --'
      end
      list[#list+1] = item
    end
  end
  if kind == "self" and self.player and self.player.achievements then
    for id, data in pairs(self.player.achievements) do
      handle(id, world.achieved[id] or {notdone=true, when="--", who="--"})
    end
  elseif kind == "main" then
    for id, data in pairs(world.achieved) do
      handle(id, data or {notdone=true, when="--", who="--"})
    end
  elseif kind == "all" then
    for _, a in ipairs(world.achiev_defs) do
      handle(a.id, world.achieved[a.id] or {notdone=true, when="--", who="--"})
    end
  end
  table.sort(list, function(a, b) return a.name < b.name end)
  self.list = list
end
